@@ -15,33 +15,25 @@ from mercurial import hg, commands, dispatch
 # don't start with a dash.  If no default value is given in the parameter list,
 # they are required.
 
+
+
 def start_hgview(ui, repo, *args, **kwargs):
-    """start hgview log viewer
-    
+    # WARNING, this docstring is superseeded programatically 
+    """
+start hgview log viewer
+=======================
+
     This command will launch the hgview log navigator, allowing to
     visually browse in the hg graph log, search in logs, and display
     diff between arbitrary revisions of a file.
 
-    Keyboard shortcuts:
+    If a filename is given, launch the filelog diff viewer for this file, 
+    and with the '-n' option, launch the filelog navigator for the file.
 
-    Up/Down     - go to next/previous revision
-    Left/Right  - display previous/next files of the current changeset
-    Ctrl+F or / - display the search bar
-    Ctrl+G      - displa the 'goto rev' bar
-    Esc         - exit
-    Enter       - run the diff viewer for the currently selected file
-                  (display diff between revisions)
-    Ctrl+R      - reread repo
+    With the '-r' option, launch the manifest viexer for the given revision.
 
-    Configuration:
-
-    Configuration statements goes under the section [hgview] of the
-    hgrc config file.
-
-    If a filename is given, only launch the filelog viewer for this file.
-    
-    Use 'hgview-options' command to display list of all configuration options.
     """
+    
     rundir = repo.root
 
     # If this user has a username validation hook enabled,
@@ -62,7 +54,8 @@ def start_hgview(ui, repo, *args, **kwargs):
         from PyQt4 import QtGui
         import hgviewlib.qt4.hgqv_rc
         from hgviewlib.qt4.hgrepoviewer import HgRepoViewer
-        from hgviewlib.qt4.hgfileviewer import FileDiffViewer
+        from hgviewlib.qt4.hgfileviewer import FileDiffViewer, FileViewer
+        from hgviewlib.qt4.hgfileviewer import ManifestViewer
     except ImportError, e:
         print e
         # If we're unable to import Qt4 and qctlib, try to
@@ -79,53 +72,31 @@ def start_hgview(ui, repo, *args, **kwargs):
         app = QtGui.QApplication(sys.argv)
         if len(args) == 1:
             # should be a filename of a file managed in the repo
-            mainwindow = FileDiffViewer(repo, args[0])
+            if kwargs.get('navigate'):
+                mainwindow = FileViewer(repo, args[0])
+            else:
+                mainwindow = FileDiffViewer(repo, args[0])
         else:
-            mainwindow = HgRepoViewer(repo)
+            rev = kwargs.get('rev')
+            if rev:
+                rev = int(rev)
+                mainwindow = ManifestViewer(repo, rev)
+            else:
+                mainwindow = HgRepoViewer(repo)
         mainwindow.show()
         return app.exec_()
 
-def display_options(ui, repo, *args, **kwargs):
-    """display hgview full list of configuration options
-    """
-    import sys
-    sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
-    from hgviewlib.config import get_option_descriptions
-    options = get_option_descriptions()
-    msg = """\nConfiguration options available for hgview.
-    These should be set under the [hgview] section.\n\n"""
-    msg += '\n'.join(["  - " + v for v in options]) + '\n'
-    msg += """
-    The 'users' config statement should be the path of a file
-    describing users, like:
-
-    -----------------------------------------------
-    # file ~/.hgusers
-    id=david
-    alias=david.douard@logilab.fr
-    alias=david@logilab.fr
-    alias=David Douard <david.douard@logilab.fr>
-    color=#FF0000
+import sys
+sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
+import hgviewlib.hgviewhelp as hghelp
     
-    id=ludal
-    alias=ludovic.aubry@logilab.fr
-    alias=ludal@logilab.fr
-    alias=Ludovic Aubry <ludovic.aubry@logilab.fr>
-    color=#00FF00
-    -----------------------------------------------
-    
-    This allow to make several 'authors' under the same name, with the
-    same color, in the graphlog browser.
-    """
-    
-    ui.status(msg)
+start_hgview.__doc__ = hghelp.long_help_msg
     
 cmdtable = {
     "^hgview|hgv|qv": (start_hgview,
-            [],
+                       [('n', 'navigate', False, '(with filename) start in navigation mode'),
+                        ('r', 'rev', '', 'start in manifest navigation mode at rev R'),
+                        ],
             "hg hgview [options] [filename]"),
-    "^hgview-options|qv-options|qv-config|qv-cfg": (display_options,
-                                     [],
-                                     "")
 }
     
