@@ -18,7 +18,7 @@ http://www.logilab.fr/ -- mailto:contact@logilab.fr
 
 import glob
 modname = 'hgview'
-numversion = (1, 1, 1)
+numversion = (1, 1, 3)
 version = '.'.join([str(num) for num in numversion])
 
 
@@ -75,7 +75,7 @@ data_files = [ #('share/hgview', ['hgviewlib/qt4/hgqv.ui',
                #               ]),
               ('share/man/man1', ['doc/hgview.1',
                                  ]),
-              ('share/python-support/mercurial-common/hgext', ['hgext/hgview.py',]),
+              #('share/python-support/mercurial-common/hgext', ['hgext/hgview.py',]),
               #('../etc/mercurial/hgrc.d', ['hgext/hgview.rc',]),
               ]
 
