@@ -22,6 +22,7 @@ Module for managing configuration parameters of hgview using Hg's
 configuration system
 """
 import os
+import re
 
 def cached(meth):
     """
@@ -39,7 +40,7 @@ def cached(meth):
 
 class HgConfig(object):
     """
-    Class managing user configuration from hg standard configuration system (.hgrc) 
+    Class managing user configuration from hg standard configuration system (.hgrc)
     """
     def __init__(self, ui, section="hgview"):
         self.ui = ui
@@ -51,8 +52,14 @@ class HgConfig(object):
         """
         font: default font used to display diffs and files. Use Qt4 format.
         """
-        return self.ui.config(self.section, 'font',
-                              'Monospace,10,-1,5,50,0,0,0,1,0')
+        return self.ui.config(self.section, 'font', 'Monospace')
+
+    @cached
+    def getFontSize(self, default=9):
+        """
+        fontsize: text size in file content viewer
+        """
+        return int(self.ui.config(self.section, 'fontsize', default))
 
     @cached
     def getDotRadius(self, default=8):
@@ -75,8 +82,7 @@ class HgConfig(object):
         if usersfile:
             try:
                 cfgfile = open(os.path.expanduser(usersfile))
-            except IOError, e:
-                #print "Cannot open file %s: please configure the 'users' parameter of the '[hgview]' section in your .hgrc file" % usersfile
+            except IOError:
                 cfgfile = None
 
         if cfgfile:
@@ -94,7 +100,8 @@ class HgConfig(object):
                 elif cmd == "alias":
                     users[currid]['aliases'].add(val)
                     if val in aliases:
-                        print "W: alias %s is used in several user definitions" % val
+                        print ("W: alias %s is used in several "
+                               "user definitions" % val)
                     aliases[val] = currid
                 else:
                     users[currid][cmd] = val
@@ -146,6 +153,81 @@ class HgConfig(object):
         """
         return int(self.ui.config(self.section, 'fillingstep', default))
 
+    @cached
+    def getChangelogColumns(self, default=None):
+        """
+        changelogcolumns: ordered list of displayed columns in changelog views;
+                    defaults to ID, Branch, Log, Author, Date, Tags
+        """
+        cols = self.ui.config(self.section, 'changelogcolumns', default)
+        if cols is None:
+            return None
+        return [col.strip() for col in cols.split(',') if col.strip()]
+
+    @cached
+    def getFilelogColumns(self, default=None):
+        """
+        filelogcolumns: ordered list of displayed columns in filelog views;
+                  defaults to ID, Log, Author, Date
+        """
+        cols = self.ui.config(self.section, 'filelogcolumns', default)
+        if cols is None:
+            return None
+        return [col.strip() for col in cols.split(',') if col.strip()]
+
+    @cached
+    def getDisplayDiffStats(self, default="yes"):
+        """
+        displaydiffstats: flag controllong the appearance of the
+                    'Diff' column in a revision's file list
+        """
+        val = str(self.ui.config(self.section, 'displaydiffstats', default))
+        return val.lower() in ['true', 'yes', '1', 'on']
+
+    @cached
+    def getMaxFileSize(self, default=100000):
+        """
+        maxfilesize: max size of a file (for diff computations, display content, etc.)
+        """
+        return int(self.ui.config(self.section, 'maxfilesize', default))
+
+    @cached
+    def getDiffBGColor(self, default='black'):
+        """
+        diffbgcolor: background color of diffs
+        """
+        return self.ui.config(self.section, 'diffbgcolor', default)
+
+    @cached
+    def getDiffFGColor(self, default='white'):
+        """
+        difffgcolor: text color of diffs
+        """
+        return self.ui.config(self.section, 'difffgcolor', default)
+
+    @cached
+    def getDiffPlusColor(self, default='green'):
+        """
+        diffpluscolor: text color of added lines in diffs
+        """
+        return self.ui.config(self.section, 'diffpluscolor', default)
+
+    @cached
+    def getDiffMinusColor(self, default='red'):
+        """
+        diffminuscolor: text color of removed lines in diffs
+        """
+        return self.ui.config(self.section, 'diffminuscolor', default)
+
+    @cached
+    def getDiffSectionColor(self, default='magenta'):
+        """
+        diffsectioncolor: text color of new section in diffs
+        """
+        return self.ui.config(self.section, 'diffsectioncolor', default)
+
+
+
 _HgConfig = HgConfig
 # HgConfig is instanciated only once (singleton)
 #
@@ -162,7 +244,10 @@ def HgConfig(ui):
     return _hgconfig
 
 
-def get_option_descriptions():
+def get_option_descriptions(rest=False):
+    """
+    Extract options descriptions (docstrings of HgConfig methods)
+    """
     options = []
     for attr in dir(_HgConfig):
         if attr.startswith('get'):
@@ -170,6 +255,10 @@ def get_option_descriptions():
             if callable(meth):
                 doc = meth.__doc__
                 if doc and doc.strip():
-                    options.append(doc.strip())
+                    doc = doc.strip()
+                    if rest:
+                        doc = re.sub(r' *(?P<arg>.*) *: *(?P<desc>.*)', r'``\1`` \2', doc.strip())
+                        doc = ' '.join(doc.split()) # remove \n and other multiple whitespaces
+                    options.append(doc)
     return options
 
