@@ -49,8 +49,14 @@ class HgDialogMixin(object):
         except IOError:
             modname = osp.splitext(osp.basename(uifile))[0] + "_ui"
             modname = "hgviewlib.qt4.%s" % modname
-            mod = __import__(modname, fromlist=['Ui_MainWindow'])
-            ui_class = mod.Ui_MainWindow
+            mod = __import__(modname, fromlist=['*'])
+            classnames = [x for x in dir(mod) if x.startswith('Ui_')]
+            if len(classnames) == 1:
+                ui_class = getattr(mod, classnames[0])
+            elif 'Ui_MainWindow' in classnames:
+                ui_class = getattr(mod, 'Ui_MainWindow')
+            else:
+                raise ValueError("Can't determine which main class to use in %s" % modname) 
         if ui_class not in self.__class__.__bases__:
             # hacking by adding the form class from ui file or pyuic4
             # generated module because we cannot use metaclass here,
@@ -58,6 +64,7 @@ class HgDialogMixin(object):
             self.__class__.__bases__ = self.__class__.__bases__ + (ui_class,)
         self.setupUi(self)
         self.load_ui()
+        self.disab_shortcuts = [self.esc_shortcut]
         
     def load_ui(self):
         # we explicitely create a QShortcut so we can disable it
@@ -73,11 +80,15 @@ class HgDialogMixin(object):
         qbar.setParent(self)
         self._quickbars.append(qbar)
         connect(qbar, SIGNAL('escShortcutDisabled(bool)'),
-                self.esc_shortcut.setEnabled)
+                self.setShortcutsEnabled)
         self.addToolBar(Qt.BottomToolBarArea, qbar)
         connect(qbar, SIGNAL('visible'),
                 self.ensureOneQuickBar)
 
+    def setShortcutsEnabled(self, enabled=True):
+        for sh in self.disab_shortcuts:
+            sh.setEnabled(enabled)
+        
     def ensureOneQuickBar(self):
         tb = self.sender()
         for w in self._quickbars:
