@@ -23,13 +23,15 @@ import os.path as osp
 
 import difflib
 
-from mercurial import ui, hg
+from mercurial import ui, hg, util
 from mercurial.node import hex, short as short_hex
 from mercurial.revlog import LookupError
 
 from PyQt4 import QtGui, QtCore, Qsci
 from PyQt4.QtCore import Qt
 
+from hgviewlib.util import tounicode
+
 from hgviewlib.qt4 import icon as geticon
 from hgviewlib.qt4.hgdialogmixin import HgDialogMixin
 from hgviewlib.qt4.hgrepomodel import FileRevModel, ManifestModel
@@ -38,6 +40,7 @@ from hgviewlib.qt4.lexers import get_lexer
 from hgviewlib.qt4.quickbar import FindInGraphlogQuickBar
 
 connect = QtCore.QObject.connect
+disconnect = QtCore.QObject.disconnect
 SIGNAL = QtCore.SIGNAL
 nullvariant = QtCore.QVariant()
 
@@ -47,21 +50,33 @@ otherside = {'left': 'right', 'right': 'left'}
 
 class FileViewer(QtGui.QMainWindow, HgDialogMixin):
     _uifile = 'fileviewer.ui'
-    def __init__(self, repo, filename, noderev=None):
+    def __init__(self, repo, filename):
+        """
+        A dialog showing a revision graph for a file.
+        """
         self.repo = repo
-        self.rev = noderev
         QtGui.QMainWindow.__init__(self)
         HgDialogMixin.__init__(self)
 
         # hg repo
         self.filename = filename
         self.createActions()
-        self.createToolbars()
+        self.setupToolbars()
 
         self.textView.setFont(self._font)
+        connect(self.textView, SIGNAL('showMessage'),
+                self.statusBar().showMessage)
         self.setupModels()
-        
-    def createToolbars(self):
+
+        connect(self.filerevmodel, SIGNAL('filled(int)'),
+                self.filled)
+
+    def filled(self, nfilled):
+        self.tableView_revisions.setCurrentIndex(self.filerevmodel.index(0,0))
+        disconnect(self.filerevmodel, SIGNAL('filled(int)'),
+                   self.filled)
+
+    def setupToolbars(self):
         self.find_toolbar = FindInGraphlogQuickBar(self)
         self.find_toolbar.attachFileView(self.textView)
         connect(self.find_toolbar, SIGNAL('revisionSelected'),
@@ -70,13 +85,19 @@ class FileViewer(QtGui.QMainWindow, HgDialogMixin):
                 self.statusBar().showMessage)
         self.attachQuickBar(self.find_toolbar)
 
+        self.toolBar_edit.addSeparator()
         self.toolBar_edit.addAction(self.tableView_revisions._actions['back'])
         self.toolBar_edit.addAction(self.tableView_revisions._actions['forward'])
+        self.toolBar_edit.addSeparator()
+        self.toolBar_edit.addAction(self.actionDiffMode)
+        self.toolBar_edit.addAction(self.actionNextDiff)
+        self.toolBar_edit.addAction(self.actionPrevDiff)
 
         self.attachQuickBar(self.tableView_revisions.goto_toolbar)
-        
+
+
     def setupModels(self):
-        self.filerevmodel = FileRevModel(self.repo, self.filename, self.rev)
+        self.filerevmodel = FileRevModel(self.repo, self.filename)
         self.tableView_revisions.setModel(self.filerevmodel)
         self.connect(self.tableView_revisions,
                      SIGNAL('revisionSelected'),
@@ -86,7 +107,7 @@ class FileViewer(QtGui.QMainWindow, HgDialogMixin):
         self.find_toolbar.setModel(self.filerevmodel)
         self.find_toolbar.setFilterFiles([self.filename])
         self.find_toolbar.setMode('file')
-        
+
     def createActions(self):
         connect(self.actionClose, SIGNAL('triggered()'),
                 self.close)
@@ -95,16 +116,48 @@ class FileViewer(QtGui.QMainWindow, HgDialogMixin):
         self.actionClose.setIcon(geticon('quit'))
         self.actionReload.setIcon(geticon('reload'))
 
+        self.actionDiffMode = QtGui.QAction('Diff mode', self)
+        self.actionDiffMode.setCheckable(True)
+        connect(self.actionDiffMode, SIGNAL('toggled(bool)'),
+                self.setMode)
+
+        self.actionNextDiff = QtGui.QAction(geticon('down'), 'Next diff', self)
+        self.actionNextDiff.setShortcut('Alt+Down')
+        self.actionPrevDiff = QtGui.QAction(geticon('up'), 'Previous diff', self)
+        self.actionPrevDiff.setShortcut('Alt+Up')
+        connect(self.actionNextDiff, SIGNAL('triggered()'),
+                self.nextDiff)
+        connect(self.actionPrevDiff, SIGNAL('triggered()'),
+                self.prevDiff)
+
+    def setMode(self, mode):
+        self.textView.setMode(mode)
+        self.actionNextDiff.setEnabled(not mode)
+        self.actionPrevDiff.setEnabled(not mode)
+
+    def nextDiff(self):
+        notlast = self.textView.nextDiff()
+        self.actionNextDiff.setEnabled(self.textView.fileMode() and notlast and self.textView.nDiffs())
+        self.actionPrevDiff.setEnabled(self.textView.fileMode() and self.textView.nDiffs())
+
+    def prevDiff(self):
+        notfirst = self.textView.prevDiff()
+        self.actionPrevDiff.setEnabled(self.textView.fileMode() and notfirst and self.textView.nDiffs())
+        self.actionNextDiff.setEnabled(self.textView.fileMode() and self.textView.nDiffs())
+
     def reload(self):
         self.repo = hg.repository(self.repo.ui, self.repo.root)
-        self.setupModels()        
-        
+        self.setupModels()
+
     def revisionSelected(self, rev):
         pos = self.textView.verticalScrollBar().value()
         ctx = self.filerevmodel.repo.changectx(rev)
         self.textView.setContext(ctx)
-        self.textView.displayFile(self.filename)
+        self.textView.displayFile(self.filerevmodel.graph.filename(rev))
         self.textView.verticalScrollBar().setValue(pos)
+        self.actionPrevDiff.setEnabled(False)
+        connect(self.textView, SIGNAL('filled'),
+                lambda self=self: self.actionNextDiff.setEnabled(self.textView.fileMode() and self.textView.nDiffs()))
 
         
 class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
@@ -123,21 +176,25 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
         self.rev = noderev
         self.setupModels()
 
-        self.createActions()        
+        self.createActions()
         self.setupTextview()
 
+    def load_config(self):
+        cfg = HgDialogMixin.load_config(self)
+        self.max_file_size = cfg.getMaxFileSize()
+
     def setupModels(self):
         self.treemodel = ManifestModel(self.repo, self.rev)
         self.treeView.setModel(self.treemodel)
         connect(self.treeView.selectionModel(),
                 SIGNAL('currentChanged(const QModelIndex &, const QModelIndex &)'),
                 self.fileSelected)
-        
+
     def createActions(self):
         connect(self.actionClose, SIGNAL('triggered()'),
                 self.close)
         self.actionClose.setIcon(geticon('quit'))
-        
+
     def setupTextview(self):
         lay = QtGui.QHBoxLayout(self.mainFrame)
         lay.setSpacing(0)
@@ -151,7 +208,7 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
 
         sci.SendScintilla(sci.SCI_SETSELEOLFILLED, True)
         self.textView = sci
-        
+
     def fileSelected(self, index, *args):
         if not index.isValid():
             return
@@ -163,17 +220,21 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
             self.textView.setMarginWidth(1, '00')
             self.textView.setText('')
             return
-        
-        if fc.size() > 100000: # XXX how to detect binary files?
-            data = "File too big"
+
+        if fc.size() > self.max_file_size:
+            data = "file too big"
         else:
             # return the whole file
-            data = unicode(fc.data(), errors='ignore') # XXX
-            lexer = get_lexer(path, data)
-            if lexer:
-                lexer.setFont(self._font)
-                self.textView.setLexer(lexer)
-            self._cur_lexer = lexer
+            data = fc.data()
+            if util.binary(data):
+                data = "binary file"
+            else:
+                data = tounicode(data)
+                lexer = get_lexer(path, data)
+                if lexer:
+                    lexer.setFont(self._font)
+                    self.textView.setLexer(lexer)
+                self._cur_lexer = lexer
         nlines = data.count('\n')
         self.textView.setMarginWidth(1, str(nlines)+'00')
         self.textView.setText(data)
@@ -189,10 +250,10 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin):
                     index = newindex
                     break
         self.treeView.setCurrentIndex(index)
-                
-        
-        
-    
+
+
+
+
 class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
     """
     Qt4 dialog to display diffs between different mercurial revisions of a file.
@@ -202,8 +263,9 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         self.repo = repo
         QtGui.QMainWindow.__init__(self)
         HgDialogMixin.__init__(self)
-        
+
         self.createActions()
+        self.setupToolbars()
         # hg repo
         self.filename = filename
         self.findLexer()
@@ -213,14 +275,14 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         # timer used to fill viewers with diff block markers during GUI idle time
         self.timer = QtCore.QTimer()
         self.timer.setSingleShot(False)
-        self.connect(self.timer, QtCore.SIGNAL("timeout()"),
-                     self.idle_fill_files)
+        connect(self.timer, SIGNAL("timeout()"),
+                self.idle_fill_files)
         self.setupModels()
-        
+
     def reload(self):
         self.repo = hg.repository(self.repo.ui, self.repo.root)
-        self.setupModels()        
-        
+        self.setupModels()
+
     def setupModels(self):
         self.filedata = {'left': None, 'right': None}
         self._invbarchanged = False
@@ -243,7 +305,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
             lexer.setDefaultFont(self._font)
             lexer.setFont(self._font)
         self.lexer = lexer
-        
+
     def createActions(self):
         connect(self.actionClose, SIGNAL('triggered()'),
                 self.close)
@@ -252,6 +314,33 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         self.actionClose.setIcon(geticon('quit'))
         self.actionReload.setIcon(geticon('reload'))
 
+        self.actionNextDiff = QtGui.QAction(geticon('down'), 'Next diff', self)
+        self.actionNextDiff.setShortcut('Alt+Down')
+        self.actionPrevDiff = QtGui.QAction(geticon('up'), 'Previous diff', self)
+        self.actionPrevDiff.setShortcut('Alt+Up')
+        connect(self.actionNextDiff, SIGNAL('triggered()'),
+                self.nextDiff)
+        connect(self.actionPrevDiff, SIGNAL('triggered()'),
+                self.prevDiff)
+        self.actionNextDiff.setEnabled(False)
+        self.actionPrevDiff.setEnabled(False)
+
+    def setupToolbars(self):
+        self.toolBar_edit.addSeparator()
+        self.toolBar_edit.addAction(self.actionNextDiff)
+        self.toolBar_edit.addAction(self.actionPrevDiff)
+
+    def setDiffNavActions(self, pos=0):
+        hasdiff = (self.diffblock.nDiffs() > 0)
+        self.actionNextDiff.setEnabled(hasdiff and pos != 1)
+        self.actionPrevDiff.setEnabled(hasdiff and pos != -1)
+
+    def nextDiff(self):
+        self.setDiffNavActions(self.diffblock.nextDiff())
+
+    def prevDiff(self):
+        self.setDiffNavActions(self.diffblock.prevDiff())
+
     def modelFilled(self):
         self.set_init_selections()
 
@@ -276,6 +365,8 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
             if self._diff is None or not self._diff.get_opcodes():
                 self._diff = None
                 self.timer.stop()
+                self.setDiffNavActions(-1)
+                self.emit(SIGNAL('diffFilled'))
                 break
 
             tag, alo, ahi, blo, bhi = self._diff.get_opcodes().pop(0)
@@ -324,7 +415,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
             self.viewers[side].setUpdatesEnabled(True)
             self.block[side].setUpdatesEnabled(True)
         self.diffblock.setUpdatesEnabled(True)
-        
+
     def update_diff(self, keeppos=None):
         """
         Recompute the diff, display files and starts the timer
@@ -333,7 +424,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         if keeppos:
             pos = self.viewers[keeppos].verticalScrollBar().value()
             keeppos = (keeppos, pos)
-            
+
         for side in sides:
             self.viewers[side].clear()
             self.block[side].clear()
@@ -360,7 +451,6 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         self.tableView_revisions_left.setCurrentIndex(self.filerevmodel.index(1, 0))
         self.tableView_revisions_right.setCurrentIndex(self.filerevmodel.index(0, 0))
 
-
     def vbar_changed(self, value, side):
         """
         Callback called when the vertical scrollbar of a file viewer
@@ -399,7 +489,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
 
     def revisionActivated(self, rev):
         """
-        Callback called when a revision is double-clicked in the revisions table        
+        Callback called when a revision is double-clicked in the revisions table
         """
         dlg = ManifestViewer(self.repo, rev)
         dlg.setCurrentFile(self.filename)
@@ -430,7 +520,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
 
             sci.setReadOnly(True)
             lay.addWidget(sci)
-            
+
             # hide margin 0 (markers)
             sci.SendScintilla(sci.SCI_SETMARGINTYPEN, 0, 0)
             sci.SendScintilla(sci.SCI_SETMARGINWIDTHN, 0, 0)
@@ -446,7 +536,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
             sci.SendScintilla(sci.SCI_MARKERSETBACK, self.markerminus, 0xA0A0FF)
             self.markertriangle = sci.markerDefine(Qsci.QsciScintilla.Background)
             sci.SendScintilla(sci.SCI_MARKERSETBACK, self.markertriangle, 0xFFA0A0)
-            
+
             self.viewers[side] = sci
             blk = BlockList(self.frame)
             blk.linkScrollBar(sci.verticalScrollBar())
@@ -458,7 +548,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
         for side in sides:
             table = getattr(self, 'tableView_revisions_%s' % side)
             table.setTabKeyNavigation(False)
-            #table.installEventFilter(self)        
+            #table.installEventFilter(self)
             connect(table, SIGNAL('revisionSelected'), self.revisionSelected)
             connect(table, SIGNAL('revisionActivated'), self.revisionActivated)
 
@@ -469,7 +559,7 @@ class FileDiffViewer(QtGui.QMainWindow, HgDialogMixin):
 
         self.setTabOrder(table, self.viewers['left'])
         self.setTabOrder(self.viewers['left'], self.viewers['right'])
-        
+
 if __name__ == '__main__':
     from mercurial import ui, hg
     from optparse import OptionParser
