@@ -28,6 +28,7 @@ from mercurial.revlog import LookupError
 from hgviewlib.hggraph import Graph, ismerge, diff as revdiff
 from hgviewlib.hggraph import revision_grapher, filelog_grapher
 from hgviewlib.config import HgConfig
+from hgviewlib.util import tounicode
 from hgviewlib.qt4 import icon as geticon
 from hgviewlib.decorators import timeit
 
@@ -38,7 +39,8 @@ nullvariant = QtCore.QVariant()
 
 # XXX make this better than a poor hard written list...
 COLORS = [ "blue", "darkgreen", "red", "green", "darkblue", "purple",
-           "cyan", "magenta", "darkred", "darkmagenta"]
+           "cyan", QtCore.Qt.darkYellow, "magenta", "darkred", "darkmagenta",
+           "darkcyan", "gray", "yellow", ]
 COLORS = [str(QtGui.QColor(x).name()) for x in COLORS]
 #COLORS = [str(color) for color in QtGui.QColor.colorNames()]
 
@@ -62,12 +64,13 @@ def cvrt_date(date):
     return QtCore.QDateTime.fromTime_t(int(date)).toString(QtCore.Qt.LocaleDate)
 
 # in following lambdas, ctx is a hg changectx
-_columnmap = {'ID': lambda ctx: ctx.rev(),
-              'Log': lambda ctx: unicode(ctx.description(), 'utf-8', 'replace'),
-              'Author': lambda ctx: unicode(ctx.user(), 'utf-8', 'replace'),
-              'Date': lambda ctx: cvrt_date(ctx.date()),
-              'Tags': lambda ctx: ",".join(ctx.tags()),
-              'Branch': lambda ctx: ctx.branch(),
+_columnmap = {'ID': lambda ctx, gnode: ctx.rev(),
+              'Log': lambda ctx, gnode: tounicode(ctx.description()),
+              'Author': lambda ctx, gnode: tounicode(ctx.user()),
+              'Date': lambda ctx, gnode: cvrt_date(ctx.date()),
+              'Tags': lambda ctx, gnode: ",".join(ctx.tags()),
+              'Branch': lambda ctx, gnode: ctx.branch(),
+              'Filename': lambda ctx, gnode: gnode.extra[0],
               }
 
 def auth_width(model, repo):
@@ -76,20 +79,22 @@ def auth_width(model, repo):
         return None
     return sorted(auths, cmp=lambda x,y: cmp(len(x), len(y)))[-1]
 
+
 # in following lambdas, r is a hg repo
 _maxwidth = {'ID': lambda self, r: str(len(r.changelog)),
              'Date': lambda self, r: cvrt_date(r.changectx(0).date()),
              'Tags': lambda self, r: sorted(r.tags().keys(),
-                                            key=lambda x: len(x))[-1],
+                                            key=lambda x: len(x))[-1][:10],
              'Branch': lambda self, r: sorted(r.branchtags().keys(),
                                               key=lambda x: len(x))[-1],
-             'Author': auth_width,
+             'Author': lambda self, r: 'author name',
+             'Filename': lambda self, r: self.filename,
              }
 
 def datacached(meth):
     """
     decorator used to cache 'data' method of Qt models. It will *not*
-    cache nullvariant return values (so costly non-null values 
+    cache nullvariant return values (so costly non-null values
     can be computed and filled as a background process)
     """
     def data(self, index, role):
@@ -109,8 +114,10 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
     """
     Model used for displaying the revisions of a Hg *local* repository
     """
-    _columns = ('ID', 'Log', 'Author', 'Date', 'Tags', 'Branch',)
+    _allcolumns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
+    _columns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
     _stretchs = {'Log': 1, }
+    _getcolumns = "getChangelogColumns"
 
     def __init__(self, repo, branch='', parent=None):
         """
@@ -118,30 +125,33 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
         """
         QtCore.QAbstractTableModel.__init__(self, parent)
         self._datacache = {}
+        self._required = None
+        self.wd_rev = None
         self.gr_fill_timer = QtCore.QTimer()
         connect(self.gr_fill_timer, SIGNAL('timeout()'),
                 self.fillGraph)
         self.setRepo(repo, branch)
 
-    #@timeit
     def setRepo(self, repo, branch=''):
         self.repo = repo
         self._datacache = {}
         self.load_config()
 
+        self.wd_rev = self.repo.changectx(None).parents()[0].rev()
         self._user_colors = {}
         self._branch_colors = {}
         grapher = revision_grapher(self.repo, branch=branch)
-        self.graph = Graph(self.repo, grapher)
+        self.graph = Graph(self.repo, grapher, self.max_file_size)
         self.nmax = len(self.repo.changelog)
         self.heads = [self.repo.changectx(x).rev() for x in self.repo.heads()]
         self._fill_iter = None
+        self._required = self.fill_step
         self.gr_fill_timer.start()
 
     def fillGraph(self):
         step = self.fill_step
         if self._fill_iter is None:
-            self.emit(SIGNAL('filling(int)'), self.nmax)
+            self.emit(SIGNAL('showMessage'), 'filling...')            
             self._fill_iter = self.graph.fill(step=step)
             self.emit(SIGNAL('layoutChanged()'))
             QtGui.QApplication.processEvents()
@@ -150,15 +160,30 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
             nm = min(n+step, self.nmax)
             self.beginInsertRows(QtCore.QModelIndex(), n, nm)
             nfilled = self._fill_iter.next()
-            self.emit(SIGNAL('filled(int)'), nfilled)
+            if n == 0:
+                self.emit(SIGNAL('filled'))                
+            if self._required and len(self.graph) > self._required:
+                self._requiredss = None
+                self.gr_fill_timer.stop()
+                self.emit(SIGNAL('showMessage'), '')            
         except StopIteration:
             self.gr_fill_timer.stop()
             self._fill_iter = None
-            self.emit(SIGNAL('fillingover()'))
+            self.emit(SIGNAL('showMessage'), '')            
         finally:
             self.endInsertRows()
             self.emit(SIGNAL('layoutChanged()'))
-        
+
+    def ensureBuilt(self, row=None, rev=None):
+        if self._fill_iter is not None and not self.gr_fill_timer.isActive():
+            if row is not None and (len(self.graph) - row) < self.fill_step:
+                self._required = len(self.graph) + self.fill_step
+            elif rev is not None and self.graph[-1].rev > rev:
+                self._required = len(self.graph) + self.fill_step + self.graph[-1].rev - rev
+            if self._required is not None:
+                self.gr_fill_timer.start()
+                self.emit(SIGNAL('showMessage'), 'filling...')            
+
     def rowCount(self, parent=None):
         return len(self.graph)
 
@@ -171,6 +196,21 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
         self.dot_radius = cfg.getDotRadius(default=8)
         self.rowheight = cfg.getRowHeight()
         self.fill_step = cfg.getFillingStep()
+        self.max_file_size = cfg.getMaxFileSize()
+        
+        cols = getattr(cfg, self._getcolumns)()
+        if cols is not None:
+            validcols = [col for col in cols if col in self._allcolumns]
+            if len(validcols) != len(cols):
+                wrongcols = [col for col in cols if col not in self._allcolumns]
+                print "WARNING! %s are not valid column names. Check your configuration." % ','.join(wrongcols)
+                print "         reverting to default columns configuration"
+            elif 'Log' not in validcols or 'ID' not in validcols:
+                print "WARNING! 'Log' and 'ID' are mandatory. Check your configuration."
+                print "         reverting to default columns configuration"
+
+            else:
+                self._columns = tuple(validcols)
 
     def maxWidthValueForColumn(self, column):
         column = self._columns[column]
@@ -209,15 +249,23 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
         if not index.isValid():
             return nullvariant
         row = index.row()
+        self.ensureBuilt(row=row)
         column = self._columns[index.column()]
         gnode = self.graph[row]
         ctx = self.repo.changectx(gnode.rev)
         if role == QtCore.Qt.DisplayRole:
             if column == 'Author': #author
-                return QtCore.QVariant(self.user_name(_columnmap[column](ctx)))
-            return QtCore.QVariant(_columnmap[column](ctx))
+                return QtCore.QVariant(self.user_name(_columnmap[column](ctx, gnode)))
+            elif column == 'Log':
+                msg = _columnmap[column](ctx, gnode)
+                if gnode.rev in self.heads:
+                    msg = "[%s] " % ctx.branch() + msg
+                return QtCore.QVariant(msg)
+            return QtCore.QVariant(_columnmap[column](ctx, gnode))
         elif role == QtCore.Qt.ToolTipRole:
-            return QtCore.QVariant(_columnmap[column](ctx))
+            msg = "<b>[%s]</b><br>\n" % ctx.branch()
+            msg += _columnmap[column](ctx, gnode)
+            return QtCore.QVariant(msg)
         elif role == QtCore.Qt.ForegroundRole:
             if column == 'Author': #author
                 return QtCore.QVariant(QtGui.QColor(self.user_color(ctx.user())))
@@ -241,9 +289,8 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
                 pen.setWidth(2)
                 painter.setPen(pen)
 
-                color = "black"
                 lpen = QtGui.QPen(pen)
-                lpen.setColor(QtGui.QColor(color))
+                lpen.setColor(QtCore.Qt.black)
                 painter.setPen(lpen)
 
                 for y1, y2, lines in ((0, h, gnode.bottomlines),
@@ -256,16 +303,26 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
                         x1 = self.col2x(start)
                         x2 = self.col2x(end)
                         painter.drawLine(x1, dot_y + y1, x2, dot_y + y2)
+
+                dot_color = QtGui.QColor(self.namedbranch_color(ctx.branch()))
+                dotcolor = QtGui.QColor(dot_color)
                 if gnode.rev in self.heads:
-                    dot_color = "yellow"
+                    penradius = 2
+                    pencolor = dotcolor.darker()
                 else:
-                    dot_color = QtGui.QColor(self.namedbranch_color(ctx.branch()))
+                    penradius = 1
+                    pencolor = QtCore.Qt.black
 
                 dot_y = (h/2) - radius / 2
 
-                painter.setBrush(QtGui.QColor(dot_color))
-                painter.setPen(QtCore.Qt.black)
-                painter.drawEllipse(dot_x, dot_y, radius, radius)
+                painter.setBrush(dotcolor)
+                pen = QtGui.QPen(pencolor)
+                pen.setWidth(penradius)
+                painter.setPen(pen)
+                if gnode.rev == self.wd_rev:
+                    painter.drawRect(dot_x, dot_y, radius, radius)
+                else:
+                    painter.drawEllipse(dot_x, dot_y, radius, radius)
                 painter.end()
                 ret = QtCore.QVariant(pix)
                 return ret
@@ -277,12 +334,13 @@ class HgRepoListModel(QtCore.QAbstractTableModel):
         return nullvariant
 
     def rowFromRev(self, rev):
-        for row, gnode in enumerate(self.graph):
-            if gnode.rev == rev:
-                return row
-        return None
+        row = self.graph.index(rev)
+        if row == -1:
+            row = None
+        return row
 
     def indexFromRev(self, rev):
+        self.ensureBuilt(rev=rev)
         row = self.rowFromRev(rev)
         if row is not None:
             return self.index(row, 0)
@@ -302,15 +360,16 @@ class FileRevModel(HgRepoListModel):
     Model used to manage the list of revisions of a file, in file
     viewer of in diff-file viewer dialogs.
     """
-    _columns = ('ID', 'Log', 'Author', 'Date',)
+    _allcolumns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags', 'Filename')
+    _columns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Filename')
     _stretchs = {'Log': 1, }
+    _getcolumns = "getFilelogColumns"
 
-    def __init__(self, repo, filename, noderev=None, parent=None):
+    def __init__(self, repo, filename, parent=None):
         """
         data is a HgHLRepo instance
         """
         HgRepoListModel.__init__(self, repo, parent=parent)
-        self.filelog = self.repo.file(filename)
         self.setFilename(filename)
 
     def setRepo(self, repo, branch=''):
@@ -320,15 +379,17 @@ class FileRevModel(HgRepoListModel):
 
     def setFilename(self, filename):
         self.filename = filename
-        self.filelog = self.repo.file(filename)
         self.nmax = len(self.repo.changelog)
-        #self.nmax = len(self.filelog)
         self._user_colors = {}
         self._branch_colors = {}
-        
+
         grapher = filelog_grapher(self.repo, self.filename)
-        self.graph = Graph(self.repo, grapher)
-        self.heads = [self.repo.changectx(x).rev() for x in self.repo.heads()]
+        self.graph = Graph(self.repo, grapher, self.max_file_size)
+        fl = self.repo.file(self.filename)
+        # we use fl.index here (instead of linkrev) cause
+        # linkrev API changed between 1.0 and 1.?. So this
+        # works with both versions.
+        self.heads = [fl.index[fl.rev(x)][4] for x in fl.heads()]
         self._datacache = {}
         self._fill_iter = None
         self.gr_fill_timer.start()
@@ -354,12 +415,12 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         self.diffwidth = 100
         self._fulllist = False
         self._fill_iter = None
-        
+
     def toggleFullFileList(self):
         self._fulllist = not self._fulllist
-        self.loadFiles()        
+        self.loadFiles()
         self.emit(SIGNAL('layoutChanged()'))
-        
+
     def load_config(self):
         cfg = HgConfig(self.repo.ui)
         self._flagcolor = {}
@@ -367,6 +428,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         self._flagcolor['-'] = cfg.getFileRemovedColor(default='red')
         self._flagcolor['-'] = cfg.getFileDeletedColor(default='red')
         self._flagcolor['+'] = cfg.getFileAddedColor(default='green')
+        self._displaydiff = cfg.getDisplayDiffStats()
 
     def setDiffWidth(self, w):
         if w != self.diffwidth:
@@ -378,12 +440,12 @@ class HgFileListModel(QtCore.QAbstractTableModel):
 
     def __len__(self):
         return len(self._files)
- 
+
     def rowCount(self, parent=None):
         return len(self)
 
     def columnCount(self, parent=None):
-        return 2
+        return 1 + self._displaydiff
 
     def file(self, row):
         return self._files[row]['path']
@@ -395,19 +457,31 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         if ctx is None:
             return self._filesdict[fn]['parent']
         return ctx.parents()[0]
-    
+
     def fileFromIndex(self, index):
         if not index.isValid() or index.row()>=len(self) or not self.current_ctx:
             return None
         row = index.row()
         return self._files[row]['path']
 
+    def revFromIndex(self, index):
+        if self._fulllist and ismerge(self.current_ctx):
+            if not index.isValid() or index.row()>=len(self) or not self.current_ctx:
+                return None
+            row = index.row()
+            current_file_desc = self._files[row]
+            if current_file_desc['fromside'] == 'right':
+                return self.current_ctx.parents()[1].rev()
+            else:
+                return self.current_ctx.parents()[0].rev()                
+        return None
+    
     def indexFromFile(self, filename):
         if filename in self._filesdict:
             row = self._files.index(self._filesdict[filename])
             return self.index(row, 0)
         return QtCore.QModelIndex()
-            
+
     def _filterFile(self, filename):
         if self._fulllist:
             return True
@@ -417,9 +491,8 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         _files = []
         ctx = self.current_ctx
         ctxfiles = ctx.files()
-        changes = self.repo.status(parent.node(), ctx.node())[:5]
-        modified, added, removed, deleted, unknown = changes
-        removed += deleted
+        changes = self.repo.status(parent.node(), ctx.node())[:3]
+        modified, added, removed = changes
         for f in [x for x in added if self._filterFile(x)]:
             desc = {'path':f, 'flag': '+', 'desc':f,
                     'parent': parent, 'fromside': fromside,
@@ -431,12 +504,12 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     removed.remove(oldname)
                     desc['renamedfrom'] = (oldname, node)
                     desc['flag'] = '='
-                    desc['desc'] += '\n(was %s)' % oldname
+                    desc['desc'] += '\n (was %s)' % oldname
                 else:
                     desc['copiedfrom'] = (oldname, node)
                     desc['flag'] = '='
                     desc['desc'] += '\n (copy of %s)' % oldname
-                    
+
             _files.append(desc)
         for f in [x for x in modified if self._filterFile(x)]:
             _files.append({'path':f, 'flag': '=', 'desc':f,
@@ -447,7 +520,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                            'parent': parent, 'fromside': fromside,
                            'infiles': f in ctxfiles})
         return _files
-    
+
     def loadFiles(self):
         self._fill_iter = None
         self._files = []
@@ -456,10 +529,10 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         if ismerge(self.current_ctx):
             _paths = [x['path'] for x in self._files]
             _files = self._buildDesc(self.current_ctx.parents()[1], 'right')
-            self._files += [x for x in _files if x['path'] not in _paths] 
+            self._files += [x for x in _files if x['path'] not in _paths]
         self._filesdict = dict([(f['path'], f) for f in self._files])
         self.fillFileStats()
-        
+
     def setSelectedRev(self, ctx):
         if ctx != self.current_ctx:
             self.current_ctx = ctx
@@ -472,8 +545,9 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         Method called to start the background process of computing
         file stats, which are to be displayed in the 'Stats' column
         """
-        self._fill_iter = self._fill()
-        self._fill_one_step()
+        if self._displaydiff:
+            self._fill_iter = self._fill()
+            self._fill_one_step()
 
     def _fill_one_step(self):
         if self._fill_iter is None:
@@ -505,14 +579,14 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     tot = max(add + rem, 1)
                 desc['stats'] = (tot, add, rem)
             yield row
-        
+
     @datacached
     def data(self, index, role):
         if not index.isValid() or index.row()>len(self) or not self.current_ctx:
             return nullvariant
         row = index.row()
         column = index.column()
-        
+
         current_file_desc = self._files[row]
         current_file = current_file_desc['path']
         stats = current_file_desc.get('stats')
@@ -552,7 +626,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     msg += "&nbsp;<b>added lines:&nbsp;</b> %s<br>" % add
                     msg += "&nbsp;<b>removed lines:&nbsp;</b> %s" % rem
                     return QtCore.QVariant(msg)
-                
+
         elif column == 0:
             if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
                 return QtCore.QVariant(current_file_desc['desc'])
@@ -584,7 +658,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                 header = ('File (merged only)', 'Diff')
         else:
             header = ('File', 'Diff')
-            
+
         if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
             return QtCore.QVariant(header[section])
 
@@ -602,7 +676,7 @@ class TreeItem(object):
         self.childItems.append(item)
         return item
     addChild = appendChild
-    
+
     def child(self, row):
         return self.childItems[row]
 
@@ -632,8 +706,8 @@ class TreeItem(object):
     def __iter__(self):
         for ch in self.childItems:
             yield ch
-            
-    
+
+
 class ManifestModel(QtCore.QAbstractItemModel):
     """
     Qt model to display a hg manifest, ie. the tree of files at a
@@ -716,7 +790,7 @@ class ManifestModel(QtCore.QAbstractItemModel):
         for path in sorted(self.changectx.manifest()):
             path = path.split(osp.sep)
             node = self.rootItem
-            
+
             for p in path:
                 for ch in node:
                     if ch.data(0) == p:
@@ -732,7 +806,7 @@ class ManifestModel(QtCore.QAbstractItemModel):
             index = self.parent(index)
         return osp.sep.join([index.internalPointer().data(0) for index in idxs])
 
-            
+
 if __name__ == "__main__":
     from mercurial import ui, hg
     from optparse import OptionParser
